library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity SDRAM_driver_test is
generic(
	COL_WIDTH     : integer                       := 9             ;--2^9 = 512 addresses in each colum
	ROW_WIDTH     : integer                       := 13            ;--2^13 = 8192 addresses in each bank
	BANK_WIDTH    : integer                       := 2             ;--2^2 = 4 banks in a single chip
	DQ_WIDTH      : integer                       := 16            ;--16 bit Data Bus for one chip
	SDRAM_MR      : std_logic_vector(15 downto 0) := X"0037"       ;--Full Page Burst,Latency=3,Burst Read and Burst Write,Standard mode
	BURST_LEN_SET : integer                       := 511            --user burst length is 512
);
port(
	clk50m : in std_logic;--external 50MHz clock input
	rst_n  : in std_logic;--reset input,active low
	
	sdram_clk   : out   std_logic                                ;--connected to the clk port of SDRAM
	sdram_cke   : out   std_logic                                ;--connected to the cke port of SDRAM
	sdram_cs_n  : out   std_logic                                ;--connected to the CS_n port of SDRAM
	sdram_ras_n : out   std_logic                                ;--connected to the RAS_n port of SDRAM
	sdram_cas_n : out   std_logic                                ;--connected to the CAS_n port of SDRAM
	sdram_we_n  : out   std_logic                                ;--connected to the WE_n port of SDRAM
	sdram_ba    : out   std_logic_vector((BANK_WIDTH-1) downto 0);--connected to the LDQM port of SDRAM
	sdram_addr  : out   std_logic_vector((ROW_WIDTH-1)  downto 0);--connected to the BA port of SDRAM
	sdram_dqm   : out   std_logic_vector((DQ_WIDTH/8-1) downto 0);--connected to the ADDR port of SDRAM
	sdram_dq    : inout std_logic_vector((DQ_WIDTH-1)   downto 0);--connected to the DQ port of SDRAM
	
	SEG : out std_logic_vector(7 downto 0)
);
end SDRAM_driver_test;

architecture bhv of SDRAM_driver_test is

	component PLL_50M_100M is
	port(
		inclk0 : in  std_logic;
		c0     : out std_logic;
		c1     : out std_logic;
		locked : out std_logic 
	);
	end component;
	
	component sdram_driver_v2 is
	generic(
		COL_WIDTH  : integer                      ;--2^8 = 256 addresses in each colum
		ROW_WIDTH  : integer                      ;--2^12 = 4096 addresses in each bank
		BANK_WIDTH : integer                      ;--2^2 = 4 banks in a single chip
		DQ_WIDTH   : integer                      ;--8 bit Data Bus for one chip
		SDRAM_MR   : std_logic_vector(15 downto 0) --Full Page Burst,Latency=3,Burst Read and Burst Write,Standard mode
	);
	port(
		clk0  : in std_logic;--The main clock of the control system
		clk1  : in std_logic;--have the same period of clk, but the phase is shifted
		rst_n : in std_logic;
		 
		user_command_req  : in  std_logic                                                    ;--The user want to send a read/write command,active HIGH
		user_command_gnt  : out std_logic                                                    ;--The state machine have accept the user's read/write command
		user_rd0_wr1      : in  std_logic                                                    ;--HIGH voltage means read and a low means write,synchronous to "user_command_req"
		user_burst_length : in  std_logic_vector((COL_WIDTH-1) downto 0)                     ;--Express the read/write lenghth of each command,synchronous to "user_command_req"
		user_start_addr   : in  std_logic_vector((BANK_WIDTH+ROW_WIDTH+COL_WIDTH-1) downto 0);--the user's read/write address,synchronous to "user_command_req"
		Tx_fifo_rd_en     : out std_logic                                                    ;--used in the user's write command,this port is synchronous to "clk"
		Tx_fifo_rd_data   : in  std_logic_vector((DQ_WIDTH/8+DQ_WIDTH-1) downto 0)           ;--the user write data to the fifo,synchronous to "clk"
		
		sdram_data_valid : out std_logic                              ;--tell the user that valid read data is coming,active HIGH
		sdram_rd_data    : out std_logic_vector((DQ_WIDTH-1) downto 0);--synchronous to sdram_data_valid
		
		sdram_clk   : out   std_logic                                ;--connected to the CLK port of SDRAM  
		sdram_cke   : out   std_logic                                ;--connected to the CKE port of SDRAM  
		sdram_cs_n  : out   std_logic                                ;--connected to the CS_n port of SDRAM 
		sdram_ras_n : out   std_logic                                ;--connected to the RAS_n port of SDRAM
		sdram_cas_n : out   std_logic                                ;--connected to the CAS_n port of SDRAM
		sdram_we_n  : out   std_logic                                ;--connected to the WE_n port of SDRAM 
		sdram_ba    : out   std_logic_vector((BANK_WIDTH-1) downto 0);--connected to the BA port of SDRAM 
		sdram_addr  : out   std_logic_vector((ROW_WIDTH-1)  downto 0);--connected to the ADDR port of SDRAM   
		sdram_dqm   : out   std_logic_vector((DQ_WIDTH/8-1) downto 0);--connected to the DQM port of SDRAM 
		sdram_dq    : inout std_logic_vector((DQ_WIDTH-1)   downto 0) --connected to the DQ port of SDRAM
	);
	end component;

	component SDRAM_test_list is
	generic(
		COL_WIDTH     : integer;--2^8 = 256 addresses in each colum
		ROW_WIDTH     : integer;--2^12 = 4096 addresses in each bank
		BANK_WIDTH    : integer;--2^2 = 4 banks in a single chip
		DQ_WIDTH      : integer;--8 bit Data Bus for one chip
		BURST_LEN_SET : integer --user burst length is 256
	);
	port(
		clk  : in std_logic;
		rst_n: in std_logic;
		
		sdram_data_valid: in std_logic                              ;--the SDRAM driver use this signal to tell the user valid read data is coming
		sdram_rd_data   : in std_logic_vector((DQ_WIDTH-1) downto 0);--the SDRAM read data
		
		user_command_req     : out    std_logic                                                    ;--The user want to send a read/write command,active HIGH
		user_command_gnt     : in     std_logic                                                    ;--The state machine have accept the user's read/write command
		user_rd0_wr1         : out    std_logic                                                    ;--HIGH voltage means read and a low means write,synchronous to "user_command_req"
		user_burst_length    : buffer std_logic_vector((COL_WIDTH-1) downto 0)                     ;--Express the read/write lenghth of each command,synchronous to "user_command_req"
		user_start_addr      : buffer std_logic_vector((BANK_WIDTH+ROW_WIDTH+COL_WIDTH-1) downto 0);--the user's read/write address,synchronous to "user_command_req"
		Tx_fifo_wr_en        : buffer std_logic                                                    ;--used in the user's write command,this port is synchronous to "clk"
		Tx_fifo_wr_data      : out    std_logic_vector(DQ_WIDTH/8+DQ_WIDTH-1 downto 0)             ;--the user write data to the fifo,synchronous to "clk"
		Tx_fifo_almost_full  : in     std_logic                                                    ;--the fifo is almost full
		Tx_fifo_almost_empty : in     std_logic                                                    ;--fifo is almost empty
		
		test_finish : out std_logic;--test finished
		test_error  : out std_logic --a error happen
	);
	end component;
	
	component Tx_fifo is
	port(
		clock        : in  STD_LOGIC ;
		data         : in  STD_LOGIC_VECTOR (17 DOWNTO 0);
		rdreq        : in  STD_LOGIC ;
		sclr         : in  STD_LOGIC ;
		wrreq        : in  STD_LOGIC ;
		almost_empty : out STD_LOGIC ;
		almost_full  : out STD_LOGIC ;
		q            : out STD_LOGIC_VECTOR (17 DOWNTO 0)
	);
	end component;
	
	signal rst_n_buf : std_logic;
	
	signal SCLK   : std_logic;
	signal DCLK   : std_logic;
	signal locked : std_logic;
	
	signal user_command_req  : std_logic                                                    ;
	signal user_command_gnt  : std_logic                                                    ;
	signal user_rd0_wr1      : std_logic                                                    ;
	signal user_burst_length : std_logic_vector((COL_WIDTH-1) downto 0)                     ;
	signal user_start_addr   : std_logic_vector((BANK_WIDTH+ROW_WIDTH+COL_WIDTH-1) downto 0);
	signal sdram_data_valid  : std_logic                                                    ;
	signal sdram_rd_data     : std_logic_vector((DQ_WIDTH-1) downto 0)                      ;
	
	signal Tx_fifo_rd_en        : std_logic                                         ;  
	signal Tx_fifo_rd_data      : std_logic_vector((DQ_WIDTH/8+DQ_WIDTH-1) downto 0);	
	signal Tx_fifo_wr_en        : std_logic                                         ;
	signal Tx_fifo_wr_data      : std_logic_vector((DQ_WIDTH/8+DQ_WIDTH-1) downto 0);
	signal Tx_fifo_almost_full  : std_logic                                         ;
	signal Tx_fifo_almost_empty : std_logic                                         ;
	
	signal sdram_test_finish : std_logic;
	signal sdram_test_error  : std_logic;
	
begin
  
	process(SCLK)
	begin
		if SCLK'event and SCLK = '1' then
			rst_n_buf <= rst_n;
		end if;
	end process;
	
	U_PLL_50M_100M : PLL_50M_100M
	port map(
		inclk0 => clk50m,
		c0     => SCLK  ,
		c1     => DCLK  ,
		locked => locked 
	);
	
	U_sdram_driver_v2 : sdram_driver_v2
	generic map(
		COL_WIDTH  => COL_WIDTH ,
		ROW_WIDTH  => ROW_WIDTH ,
		BANK_WIDTH => BANK_WIDTH,
		DQ_WIDTH   => DQ_WIDTH  ,
		SDRAM_MR   => SDRAM_MR  
	)
	port map(
		clk0  => SCLK     ,--The main clock of the control system
		clk1  => DCLK     ,--have the same period of clk, but the phase is shifted
		rst_n => rst_n_buf,
		      
		user_command_req  => user_command_req ,--The user want to send a read/write command,active HIGH
		user_command_gnt  => user_command_gnt ,--The state machine have accept the user's read/write command
		user_rd0_wr1      => user_rd0_wr1     ,--HIGH voltage means read and a low means write,synchronous to "user_command_req"
		user_burst_length => user_burst_length,--Express the read/write lenghth of each command,synchronous to "user_command_req"
		user_start_addr   => user_start_addr  ,--the user's read/write address,synchronous to "user_command_req"
		Tx_fifo_rd_en     => Tx_fifo_rd_en    ,--used in the user's write command,this port is synchronous to "clk"
		Tx_fifo_rd_data   => Tx_fifo_rd_data  ,--the user write data to the fifo,synchronous to "clk"
		
		sdram_data_valid => sdram_data_valid,--tell the user that valid read data is coming,active HIGH
		sdram_rd_data    => sdram_rd_data   ,--synchronous to sdram_data_valid
		
		sdram_clk   => sdram_clk  ,--connected to the CLK port of SDRAM  
		sdram_cke   => sdram_cke  ,--connected to the CKE port of SDRAM  
		sdram_cs_n  => sdram_cs_n ,--connected to the CS_n port of SDRAM 
		sdram_ras_n => sdram_ras_n,--connected to the RAS_n port of SDRAM
		sdram_cas_n => sdram_cas_n,--connected to the CAS_n port of SDRAM
		sdram_we_n  => sdram_we_n ,--connected to the WE_n port of SDRAM 
		sdram_ba    => sdram_ba   ,--connected to the BA port of SDRAM 
		sdram_addr  => sdram_addr ,--connected to the ADDR port of SDRAM   
		sdram_dqm   => sdram_dqm  ,--connected to the DQM port of SDRAM 
		sdram_dq    => sdram_dq    --connected to the DQ port of SDRAM
	);
	
	U_SDRAM_test_list : SDRAM_test_list
	generic map(
		COL_WIDTH     => COL_WIDTH    ,
		ROW_WIDTH     => ROW_WIDTH    ,
		BANK_WIDTH    => BANK_WIDTH   ,
		DQ_WIDTH      => DQ_WIDTH     ,
		BURST_LEN_SET => BURST_LEN_SET 		
	)
	port map(
		clk   => SCLK     ,
		rst_n => rst_n_buf,
		
		sdram_data_valid => sdram_data_valid,--the SDRAM driver use this signal to tell the user valid read data is coming
		sdram_rd_data    => sdram_rd_data   ,--the SDRAM read data
		
		user_command_req     => user_command_req    ,--The user want to send a read/write command,active HIGH
		user_command_gnt     => user_command_gnt    ,--The state machine have accept the user's read/write command
		user_rd0_wr1         => user_rd0_wr1        ,--HIGH voltage means read and a low means write,synchronous to "user_command_req"
		user_burst_length    => user_burst_length   ,--Express the read/write lenghth of each command,synchronous to "user_command_req"
		user_start_addr      => user_start_addr     ,--the user's read/write address,synchronous to "user_command_req"
		Tx_fifo_wr_en        => Tx_fifo_wr_en       ,--used in the user's write command,this port is synchronous to "clk"
		Tx_fifo_wr_data      => Tx_fifo_wr_data     ,--the user write data to the fifo,synchronous to "clk"
		Tx_fifo_almost_full  => Tx_fifo_almost_full ,--the fifo is almost full
		Tx_fifo_almost_empty => Tx_fifo_almost_empty,--fifo is almost empty
		
		test_finish => sdram_test_finish,--test finished
		test_error  => sdram_test_error  --a error happen
	);
	
	U_Tx_fifo : Tx_fifo
	port map(
		clock        => SCLK                ,
		data         => Tx_fifo_wr_data     ,
		rdreq        => Tx_fifo_rd_en       ,
		sclr         => not rst_n_buf       ,
		wrreq        => Tx_fifo_wr_en       ,
		almost_empty => Tx_fifo_almost_empty,
		almost_full  => Tx_fifo_almost_full ,
		q            => Tx_fifo_rd_data      
	);
	
	process(SCLK)
	begin
		if SCLK'event and SCLK = '1' then
			if sdram_test_finish = '0' then
				SEG <= "10000000";--display an "."
			elsif sdram_test_error = '1' then
				SEG <= "01110001";--display a "F"
			else
				SEG <= "01101101";--display a "S"
			end if;
		end if;
	end process;

end bhv;