library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

entity sdram_test_list is
generic(
	COL_WIDTH     : integer;--2^8 = 256 addresses in each colum
	ROW_WIDTH     : integer;--2^12 = 4096 addresses in each bank
	BANK_WIDTH    : integer;--2^2 = 4 banks in a single chip
	DQ_WIDTH      : integer;--8 bit Data Bus for one chip
	BURST_LEN_SET : integer --user burst length is 256
);
port(
	clk  : in std_logic;
	rst_n: in std_logic;
	
	sdram_data_valid: in std_logic                              ;--the SDRAM driver use this signal to tell the user valid read data is coming
	sdram_rd_data   : in std_logic_vector((DQ_WIDTH-1) downto 0);--the SDRAM read data
	
	user_command_req     : out    std_logic                                                    ;--The user want to send a read/write command,active HIGH
	user_command_gnt     : in     std_logic                                                    ;--The state machine have accept the user's read/write command
	user_rd0_wr1         : out    std_logic                                                    ;--HIGH voltage means read and a low means write,synchronous to "user_command_req"
	user_burst_length    : buffer std_logic_vector((COL_WIDTH-1) downto 0)                     ;--Express the read/write lenghth of each command,synchronous to "user_command_req"
	user_start_addr      : buffer std_logic_vector((BANK_WIDTH+ROW_WIDTH+COL_WIDTH-1) downto 0);--the user's read/write address,synchronous to "user_command_req"
	Tx_fifo_wr_en        : buffer std_logic                                                    ;--used in the user's write command,this port is synchronous to "clk"
	Tx_fifo_wr_data      : buffer std_logic_vector(DQ_WIDTH/8+DQ_WIDTH-1 downto 0)             ;--the user write data to the fifo,synchronous to "clk"
	Tx_fifo_almost_full  : in     std_logic                                                    ;--the fifo is almost full
	Tx_fifo_almost_empty : in     std_logic                                                    ;--fifo is almost empty
	
	test_finish : out std_logic;--test finished
	test_error  : out std_logic --a error happen
);

end sdram_test_list;

architecture bhv of sdram_test_list is

	constant DELAY_TIME :std_logic_vector(23 downto 0) := X"FFFFFF";
	
	signal cnt : std_logic_vector(23 downto 0);
	signal cnt_en : std_logic;
	signal time_overflow : std_logic;
	
	type test_states is(
  		idle           ,
  		wait_write_gnt ,
  		write_A_row    ,
  		inc_write_row  ,
  		wait_read_gnt  ,
  		read_A_row     ,
  		inc_read_row   ,
  		begin_next_test,
  		test_finished            
	);
	signal test_state : test_states;
	
	signal A_row_write_done : std_logic;
	signal A_row_read_done  : std_logic;
	signal one_hot_addr : std_logic_vector((BANK_WIDTH+ROW_WIDTH-1) downto 0);
	signal test_phase : std_logic_vector(2 downto 0);
	
	signal expected_data : std_logic_vector(DQ_WIDTH-1 downto 0);
	
	signal burst_len_cnt : std_logic_vector((COL_WIDTH-1) downto 0);
	signal receive_data_cnt : std_logic_vector((COL_WIDTH-1) downto 0);
	
	signal receive_data_cnt_com : std_logic_vector((DQ_WIDTH-1) downto 0);
	
begin
	
	process(clk)
	begin
		if clk'event and clk = '1' then
			if rst_n = '0' then
				cnt <= (others => '0');
			elsif cnt_en = '1' then
				cnt <= cnt + '1';
			else
				cnt <= (others => '0');
			end if;
		end if;
	end process;
	
	process(clk)
	begin
		if clk'event and clk = '1' then
			if rst_n = '0' then
				time_overflow <= '0';
			elsif cnt_en = '0' then
				time_overflow <= '0';
			elsif cnt = DELAY_TIME then
				time_overflow <= '1';
			end if;
		end if;
	end process;
	
	process(clk)
	begin
		if clk'event and clk = '1' then
			if rst_n = '0' then
				test_state <= idle;
			else
				case test_state is
					when idle =>
						if time_overflow = '1' then
							test_state <= wait_write_gnt;
						end if;
					when wait_write_gnt =>
						if user_command_gnt = '1' then
							test_state <= write_A_row;
						end if;
					when write_A_row =>
						if A_row_write_done = '1' then
							case test_phase is
								when "000" =>--test the first row 
									test_state <= wait_read_gnt;
								when "001" =>--test the last row
									test_state <= wait_read_gnt;
								when "010" =>--test dq[n] pin
									test_state <= wait_read_gnt;
								when "011" =>--test mask pin
									test_state <= wait_read_gnt;
								when "100" =>--test bank/addr[n] with one hot '1'
									test_state <= inc_write_row;
								when "101" =>--test bank/addr[n] with one hot '0'
									test_state <= inc_write_row;
								when "110" =>--check the first row again
									test_state <= wait_read_gnt;
								when "111" =>--check the last row again
									test_state <= wait_read_gnt;
								when others => null;
							end case;
						end if;
					when inc_write_row =>
						if one_hot_addr(BANK_WIDTH+ROW_WIDTH-1) = '1' then
							test_state <= wait_read_gnt;
						else
							test_state <= wait_write_gnt;
						end if;
					when wait_read_gnt =>
						if user_command_gnt = '1' then
							test_state <= read_A_row;
						end if;
					when read_A_row =>
						if A_row_read_done = '1' then
							case test_phase is
								when "000" =>--test the first row 
									test_state <= begin_next_test;
								when "001" =>--test the last row
									test_state <= begin_next_test;
								when "010" =>--test dq[n] pin
									test_state <= begin_next_test;
								when "011" =>--test mask pin
									test_state <= begin_next_test;
								when "100" =>--test bank/addr[n] with one hot '1'
									test_state <= inc_read_row;
								when "101" =>--test bank/addr[n] with one hot '0'
									test_state <= inc_read_row;
								when "110" =>--check the first row again
									test_state <= begin_next_test;
								when "111" =>--check the last row again
									test_state <= begin_next_test;
								when others => null;
							end case;
						end if;
					when inc_read_row =>
						if one_hot_addr(BANK_WIDTH+ROW_WIDTH-1) = '1' then
							test_state <= begin_next_test;
						else
							test_state <= wait_read_gnt;
						end if;
					when begin_next_test =>
						if test_phase = "111" then
							test_state <= test_finished;
						else
							test_state <= wait_write_gnt;
						end if;
					when test_finished =>
						null;
					when others =>
						test_state <= idle;
				end case;
			end if;
		end if;
	end process;
	
	process(clk)
	begin
		if clk'event and clk = '1' then
			if rst_n = '0' then
				test_finish <= '0';
			elsif test_state = test_finished then
				test_finish <= '1';
			else
				test_finish <= '0';
			end if;
		end if;
	end process;
	
	process(clk)
	begin
		if clk'event and clk = '1' then
			if rst_n = '0' then
				cnt_en <= '0';
			elsif test_state = idle then
				cnt_en <= '1';
			else
				cnt_en <= '0';
			end if;
		end if;
	end process;
	
	user_burst_length <= conv_std_logic_vector(BURST_LEN_SET,COL_WIDTH);
	
	process(clk)
	begin
		if clk'event and clk = '1' then
			if rst_n = '0' then
				one_hot_addr <= (others => '0');
			else
				case test_phase is
					when "000" =>--test the first row 
						one_hot_addr <= (others => '0');
					when "001" =>--test the last row
						one_hot_addr <= (others => '1');
					when "010" =>--test dq[n] pin
						one_hot_addr <= conv_std_logic_vector(1,(BANK_WIDTH+ROW_WIDTH));
					when "011" =>--test mask pin
						one_hot_addr <= conv_std_logic_vector(1,(BANK_WIDTH+ROW_WIDTH));
					when "100" =>--test bank/addr[n] with one hot '1'
						if test_state=inc_write_row or test_state=inc_read_row then
							one_hot_addr <= one_hot_addr(BANK_WIDTH+ROW_WIDTH-2 downto 0) & one_hot_addr(BANK_WIDTH+ROW_WIDTH-1);
						end if;
					when "101" =>--test bank/addr[n] with one hot '0'
						if test_state=inc_write_row or test_state=inc_read_row then
							one_hot_addr <= one_hot_addr(BANK_WIDTH+ROW_WIDTH-2 downto 0) & one_hot_addr(BANK_WIDTH+ROW_WIDTH-1);
						end if;
					when "110" =>--check the first row again
						one_hot_addr <= (others => '0');
					when "111" =>--check the last row again
						one_hot_addr <= (others => '1');
					when others => null;
				end case;
			end if;
		end if;
	end process;
	
	process(one_hot_addr,test_phase)
	begin
		if test_phase = "101" then--test bank/addr[n] with one hot '0'
			user_start_addr <= (not one_hot_addr) & conv_std_logic_vector(0,COL_WIDTH);
		else
			user_start_addr <= one_hot_addr & conv_std_logic_vector(0,COL_WIDTH);
		end if;
	end process;
	
	process(clk)
	begin
		if clk'event and clk = '1' then
			if rst_n = '0' then
				test_phase <= (others => '0');
			elsif test_state = idle then
				test_phase <= (others => '0');
			elsif test_state = begin_next_test then
				test_phase <= test_phase + '1';
			end if;
		end if;
	end process;
	
	process(clk)
	begin
		if clk'event and clk = '1' then
			if rst_n = '0' then
				user_command_req <= '0';
			else
				case test_state is
					when wait_write_gnt |
						 wait_read_gnt  =>
							if user_command_gnt = '1' then
								user_command_req <= '0';
							else
								user_command_req <= '1';
							end if;                   
					when others =>
						user_command_req <= '0';
				end case;
			end if;
		end if;
	end process;
	
	process(clk)
	begin
		if clk'event and clk = '1' then
			if rst_n = '0' then
				user_rd0_wr1 <= '0';
			elsif test_state = wait_write_gnt then
				user_rd0_wr1 <= '1';
			elsif test_state = wait_read_gnt then
				user_rd0_wr1 <= '0';
			end if;
		end if;
	end process;
	
	process(clk)
	begin
		if clk'event and clk = '1' then
			if rst_n = '0' then
				Tx_fifo_wr_en <= '0';
			elsif test_state = write_A_row then
				Tx_fifo_wr_en <= '1';
			else
				Tx_fifo_wr_en <= '0';
			end if;
		end if;
	end process;
	
	process(clk)
	begin
		if clk'event and clk = '1' then
			if rst_n = '0' then
				burst_len_cnt <= (others => '1');
			elsif test_state = write_A_row then
				burst_len_cnt <= burst_len_cnt + '1';
			else
				burst_len_cnt <= (others => '1');
			end if;
		end if;
	end process;
	
	A_row_write_done <= '1' when (burst_len_cnt+'1' = user_burst_length) else '0';
	
	process(clk)
	begin
		if clk'event and clk = '1' then
			if rst_n = '0' then
				Tx_fifo_wr_data(DQ_WIDTH-1 downto 0) <= (others => '0');
			elsif test_state=begin_next_test then
				Tx_fifo_wr_data(DQ_WIDTH-1 downto 0) <= conv_std_logic_vector(1,DQ_WIDTH);
			else
				case test_phase is
					when "000" =>--test the first row 
						Tx_fifo_wr_data(DQ_WIDTH-1 downto 0) <= (others => '0');
					when "001" =>--test the last row
						Tx_fifo_wr_data(DQ_WIDTH-1 downto 0) <= (others => '1');
					when "010" =>--test dq[n] pin
						if Tx_fifo_wr_en = '1' then
							Tx_fifo_wr_data(DQ_WIDTH-1 downto 0) <= Tx_fifo_wr_data(DQ_WIDTH-2 downto 0) & Tx_fifo_wr_data(DQ_WIDTH-1);
						end if;
					when "011" =>--test mask pin
						Tx_fifo_wr_data(DQ_WIDTH-1 downto 0) <= (others => '0');
					when "100" =>--test bank/addr[n] with one hot '1'
						if test_state = inc_write_row then
							Tx_fifo_wr_data(DQ_WIDTH-1 downto 0) <= Tx_fifo_wr_data(DQ_WIDTH-1 downto 0) + 1;
						end if;
					when "101" =>--test bank/addr[n] with one hot '0'
						if test_state = inc_write_row then
							Tx_fifo_wr_data(DQ_WIDTH-1 downto 0) <= Tx_fifo_wr_data(DQ_WIDTH-1 downto 0) + 1;
						end if;
					when "110" =>--check the first row again
						Tx_fifo_wr_data(DQ_WIDTH-1 downto 0) <= (others => '1');
					when "111" =>--check the last row again
						Tx_fifo_wr_data(DQ_WIDTH-1 downto 0) <= (others => '0');
					when others => null;
				end case;
			end if;
		end if;
	end process;
	
	process(test_phase,Tx_fifo_wr_data)
	begin
		case test_phase is
			when "000"| --test the first row 
			     "001"| --test the last row
			     "010"| --test dq[n] pin
			     "100"| --test bank/addr[n] with one hot '1'
			     "101"=>--test bank/addr[n] with one hot '0'
				Tx_fifo_wr_data(DQ_WIDTH/8+DQ_WIDTH-1 downto DQ_WIDTH) <= (others => '0');
			when "011"|  --test mask pin
			     "110"|  --check the first row again
			     "111"=> --check the last row again
				Tx_fifo_wr_data(DQ_WIDTH/8+DQ_WIDTH-1 downto DQ_WIDTH) <= (others => '1');
			when others=>
				Tx_fifo_wr_data(DQ_WIDTH/8+DQ_WIDTH-1 downto DQ_WIDTH) <= (others => '1');
		end case;
	end process;
	
	process(clk)
	begin
		if clk'event and clk = '1' then
			if rst_n = '0' then
				receive_data_cnt <= (others => '0');
			elsif test_state = read_A_row then
				if sdram_data_valid = '1' then
					receive_data_cnt <= receive_data_cnt + '1';
				end if;
			else
				receive_data_cnt <= (others => '0');
			end if;
		end if;
	end process;
	
	process(clk)
	begin
		if clk'event and clk = '1' then
			if rst_n = '0' then
				A_row_read_done <= '0';
			elsif test_state = read_A_row then
				if sdram_data_valid = '1' and receive_data_cnt = user_burst_length then
					A_row_read_done <= '1';
				end if;
			else
				A_row_read_done <= '0';
			end if;
		end if;
	end process;
	
	process(clk)
	begin
		if clk'event and clk = '1' then
			if rst_n = '0' then
				expected_data <= (others => '0');
			elsif test_state = begin_next_test then
				expected_data <= conv_std_logic_vector(1,DQ_WIDTH);
			else
				case test_phase is
					when "000" =>--test the first row 
						expected_data <= (others => '0');
					when "001" =>--test the last row
						expected_data <= (others => '1');
					when "010" =>--test dq[n] pin
						if sdram_data_valid = '1' then
							expected_data <= expected_data(DQ_WIDTH-2 downto 0) & expected_data(DQ_WIDTH-1);
						end if;
					when "011" =>--test mask pin
						if sdram_data_valid = '1' then
							expected_data <= expected_data(DQ_WIDTH-2 downto 0) & expected_data(DQ_WIDTH-1);
						end if;
					when "100" =>--test bank/addr[n] with one hot '1'
						if test_state = inc_read_row then
							expected_data <= expected_data + 1;
						end if;
					when "101" =>--test bank/addr[n] with one hot '0'
						if test_state = inc_read_row then
							expected_data <= expected_data + 1;
						end if;
					when "110" =>--check the first row again
						expected_data <= (others => '0');
					when "111" =>--check the last row again
						expected_data <= (others => '1');
					when others => null;
				end case;
			end if;
		end if;
	end process;
	
	process(clk)
	begin
		if clk'event and clk = '1' then
			if rst_n = '0' then
				test_error <= '0';
			else
				case test_state is
					when idle =>
						test_error <= '0';
					when read_A_row =>
						if sdram_data_valid = '1' and sdram_rd_data /= expected_data then
							test_error <= '1';
						end if;
					when others => null;
				end case;
			end if;
		end if;
	end process;
	
end bhv;