library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity sdram_driver_v2 is
generic(
	COL_WIDTH  : integer                       := 8              ;--2^8 = 256 addresses in each colum
	ROW_WIDTH  : integer                       := 12             ;--2^12 = 4096 addresses in each bank
	BANK_WIDTH : integer                       := 2              ;--2^2 = 4 banks in a single chip
	DQ_WIDTH   : integer                       := 8              ;--8 bit Data Bus for one chip
	SDRAM_MR   : std_logic_vector(15 downto 0) := X"0037"         --Full Page Burst,Latency=3,Burst Read and Burst Write,Standard mode
);
port(
	clk0  : in std_logic;--The main clock of the control system
	clk1  : in std_logic;--have the same period of clk, but the phase is shifted
	rst_n : in std_logic;
	 
	user_command_req  : in  std_logic                                                    ;--The user want to send a read/write command,active HIGH
	user_command_gnt  : out std_logic                                                    ;--The state machine have accept the user's read/write command
	user_rd0_wr1      : in  std_logic                                                    ;--HIGH voltage means read and a low means write,synchronous to "user_command_req"
	user_burst_length : in  std_logic_vector((COL_WIDTH-1) downto 0)                     ;--Express the read/write lenghth of each command,synchronous to "user_command_req"
	user_start_addr   : in  std_logic_vector((BANK_WIDTH+ROW_WIDTH+COL_WIDTH-1) downto 0);--the user's read/write address,synchronous to "user_command_req"
	Tx_fifo_rd_en     : out std_logic                                                    ;--used in the user's write command,this port is synchronous to "clk"
	Tx_fifo_rd_data   : in  std_logic_vector((DQ_WIDTH/8+DQ_WIDTH-1) downto 0)           ;--the user write data to the fifo,synchronous to "clk"
	
	sdram_data_valid : out std_logic                              ;--tell the user that valid read data is coming,active HIGH
	sdram_rd_data    : out std_logic_vector((DQ_WIDTH-1) downto 0);--synchronous to sdram_data_valid
	
	sdram_clk   : out   std_logic                                ;--connected to the CLK port of SDRAM  
	sdram_cke   : out   std_logic                                ;--connected to the CKE port of SDRAM  
	sdram_cs_n  : out   std_logic                                ;--connected to the CS_n port of SDRAM 
	sdram_ras_n : out   std_logic                                ;--connected to the RAS_n port of SDRAM
	sdram_cas_n : out   std_logic                                ;--connected to the CAS_n port of SDRAM
	sdram_we_n  : out   std_logic                                ;--connected to the WE_n port of SDRAM 
	sdram_ba    : out   std_logic_vector((BANK_WIDTH-1) downto 0);--connected to the BA port of SDRAM 
	sdram_addr  : out   std_logic_vector((ROW_WIDTH-1)  downto 0);--connected to the ADDR port of SDRAM   
	sdram_dqm   : out   std_logic_vector((DQ_WIDTH/8-1) downto 0);--connected to the DQM port of SDRAM 
	sdram_dq    : inout std_logic_vector((DQ_WIDTH-1)   downto 0) --connected to the DQ port of SDRAM
);
end sdram_driver_v2;

architecture bhv of sdram_driver_v2 is
	
	component sdram_ctrl is
	generic(
		COL_WIDTH  : integer                      ;--2^8 = 256 addresses in each colum
		ROW_WIDTH  : integer                      ;--2^12 = 4096 addresses in each bank
		BANK_WIDTH : integer                      ;--2^2 = 4 banks in a single chip
		DQ_WIDTH   : integer                      ;--8 bit Data Bus for one chip
		SDRAM_MR   : std_logic_vector(15 downto 0) --user burst length is 256
	);
	port(
		clk   : in std_logic;--The main clock of the control system
		rst_n : in std_logic;--A low voltage may bring the control system to an oringinal state,but the data in SDRAM may be damaged
		
		user_command_req  : in  std_logic                                                    ;--The user want to send a read/write command,active HIGH
		user_command_gnt  : out std_logic                                                    ;--The state machine have accept the user's read/write command
		user_rd0_wr1      : in  std_logic                                                    ;--HIGH voltage means read and a low means write,synchronous to "user_command_req"
		user_burst_length : in  std_logic_vector((COL_WIDTH-1) downto 0)                     ;--Express the read/write lenghth of each command,synchronous to "user_command_req"
		user_start_addr   : in  std_logic_vector((BANK_WIDTH+ROW_WIDTH+COL_WIDTH-1) downto 0);--the user's read/write address,synchronous to "user_command_req"
		Tx_fifo_rd_en     : out std_logic                                                    ;--used in the user's write command,this port is synchronous to "clk0"
		Tx_fifo_rd_data   : in  std_logic_vector((DQ_WIDTH/8+DQ_WIDTH-1) downto 0)           ;--the user write data to the fifo,synchronous to "clk0"
		
		sdram_cke       : out std_logic                                ;
		sdram_cs_n      : out std_logic                                ;
		sdram_ras_n     : out std_logic                                ;
		sdram_cas_n     : out std_logic                                ;
		sdram_we_n      : out std_logic                                ;
		sdram_ba        : out std_logic_vector((BANK_WIDTH-1) downto 0);
		sdram_addr      : out std_logic_vector((ROW_WIDTH-1) downto 0) ;
		sdram_dq_out_en : out std_logic                                ;
		sdram_dqm       : out std_logic_vector((DQ_WIDTH/8-1) downto 0);
		sdram_dq_out    : out std_logic_vector((DQ_WIDTH-1)   downto 0);
		sdram_dq_in     : in  std_logic_vector((DQ_WIDTH-1)   downto 0);
		
		sdram_data_valid : out std_logic                              ;--tell the user that valid read data is coming,active HIGH
		sdram_rd_data    : out std_logic_vector((DQ_WIDTH-1) downto 0) --synchronous to sdram_data_valid
	);
	end component;
	
	component sdram_io is
	generic(
		ROW_WIDTH     : integer;--2^12 = 4096 addresses in each bank
		BANK_WIDTH    : integer;--2^2 = 4 banks in a single chip
		DQ_WIDTH      : integer --8 bit Data Bus for one chip
	);
	port(
		clk0 : in std_logic;
		clk1 : in std_logic;
		
		c0_sdram_cke       : in  std_logic                                ;
		c0_sdram_cs_n      : in  std_logic                                ;
		c0_sdram_ras_n     : in  std_logic                                ;
		c0_sdram_cas_n     : in  std_logic                                ;
		c0_sdram_we_n      : in  std_logic                                ;
		c0_sdram_ba        : in  std_logic_vector((BANK_WIDTH-1) downto 0);
		c0_sdram_addr      : in  std_logic_vector((ROW_WIDTH-1) downto 0) ;
		c0_sdram_dq_out_en : in  std_logic                                ;
		c0_sdram_dqm       : in  std_logic_vector((DQ_WIDTH/8-1) downto 0);
		c0_sdram_dq_out    : in  std_logic_vector((DQ_WIDTH-1)   downto 0);
		c0_sdram_dq_in     : out std_logic_vector((DQ_WIDTH-1)   downto 0);
		
		c1_sdram_clk     : out   std_logic                                ;
		c1_sdram_cke     : out   std_logic                                ;
		c1_sdram_cs_n    : out   std_logic                                ;
		c1_sdram_ras_n   : out   std_logic                                ;
		c1_sdram_cas_n   : out   std_logic                                ;
		c1_sdram_we_n    : out   std_logic                                ;
		c1_sdram_ba      : out   std_logic_vector((BANK_WIDTH-1) downto 0);
		c1_sdram_addr    : out   std_logic_vector((ROW_WIDTH-1)  downto 0);
		c1_sdram_dqm     : out   std_logic_vector((DQ_WIDTH/8-1) downto 0);
		c1_sdram_dq      : inout std_logic_vector((DQ_WIDTH-1)   downto 0) 
	);
	end component;
	
	signal c0_sdram_cke       : std_logic                                ;
	signal c0_sdram_cs_n      : std_logic                                ;
	signal c0_sdram_ras_n     : std_logic                                ;
	signal c0_sdram_cas_n     : std_logic                                ;
	signal c0_sdram_we_n      : std_logic                                ;
	signal c0_sdram_ba        : std_logic_vector((BANK_WIDTH-1) downto 0);
	signal c0_sdram_addr      : std_logic_vector((ROW_WIDTH-1) downto 0) ;
	signal c0_sdram_dq_out_en : std_logic                                ;
	signal c0_sdram_dqm       : std_logic_vector((DQ_WIDTH/8-1) downto 0);
	signal c0_sdram_dq_out    : std_logic_vector((DQ_WIDTH-1)   downto 0);
	signal c0_sdram_dq_in     : std_logic_vector((DQ_WIDTH-1)   downto 0);
	
begin
	
	u_sdram_ctrl : sdram_ctrl
	generic map(
		COL_WIDTH  => COL_WIDTH ,
		ROW_WIDTH  => ROW_WIDTH ,
		BANK_WIDTH => BANK_WIDTH,
		DQ_WIDTH   => DQ_WIDTH  ,
		SDRAM_MR   => SDRAM_MR  
	)
	port map(
		clk   => clk0 ,--The main clock of the control system
		rst_n => rst_n,--A low voltage may bring the control system to an oringinal state,but the data in SDRAM may be damaged
		
		user_command_req  => user_command_req ,--The user want to send a read/write command,active HIGH
		user_command_gnt  => user_command_gnt ,--The state machine have accept the user's read/write command
		user_rd0_wr1      => user_rd0_wr1     ,--HIGH voltage means read and a low means write,synchronous to "user_command_req"
		user_burst_length => user_burst_length,--Express the read/write lenghth of each command,synchronous to "user_command_req"
		user_start_addr   => user_start_addr  ,--the user's read/write address,synchronous to "user_command_req"
		Tx_fifo_rd_en     => Tx_fifo_rd_en    ,--used in the user's write command,this port is synchronous to "clk0"
		Tx_fifo_rd_data   => Tx_fifo_rd_data  ,--the user write data to the fifo,synchronous to "clk0"
		
		sdram_cke       => c0_sdram_cke      ,
		sdram_cs_n      => c0_sdram_cs_n     ,
		sdram_ras_n     => c0_sdram_ras_n    ,
		sdram_cas_n     => c0_sdram_cas_n    ,
		sdram_we_n      => c0_sdram_we_n     ,
		sdram_ba        => c0_sdram_ba       ,
		sdram_addr      => c0_sdram_addr     ,
		sdram_dq_out_en => c0_sdram_dq_out_en,
		sdram_dqm       => c0_sdram_dqm      ,
		sdram_dq_out    => c0_sdram_dq_out   ,
		sdram_dq_in     => c0_sdram_dq_in    ,
		
		sdram_data_valid => sdram_data_valid,--tell the user that valid read data is coming,active HIGH
		sdram_rd_data    => sdram_rd_data    --synchronous to sdram_data_valid
	);
	
	u_sdram_io : sdram_io
	generic map(
		ROW_WIDTH  => ROW_WIDTH ,
		BANK_WIDTH => BANK_WIDTH,
		DQ_WIDTH   => DQ_WIDTH  
	)
	port map(
		clk0 => clk0,
		clk1 => clk1,
	
		c0_sdram_cke       => c0_sdram_cke      ,
		c0_sdram_cs_n      => c0_sdram_cs_n     ,
		c0_sdram_ras_n     => c0_sdram_ras_n    ,
		c0_sdram_cas_n     => c0_sdram_cas_n    ,
		c0_sdram_we_n      => c0_sdram_we_n     ,
		c0_sdram_ba        => c0_sdram_ba       ,
		c0_sdram_addr      => c0_sdram_addr     ,
		c0_sdram_dq_out_en => c0_sdram_dq_out_en,
		c0_sdram_dqm       => c0_sdram_dqm      ,
		c0_sdram_dq_out    => c0_sdram_dq_out   ,
		c0_sdram_dq_in     => c0_sdram_dq_in    ,
		
		c1_sdram_clk      => sdram_clk  ,
		c1_sdram_cke      => sdram_cke  ,
		c1_sdram_cs_n     => sdram_cs_n ,
		c1_sdram_ras_n    => sdram_ras_n,
		c1_sdram_cas_n    => sdram_cas_n,
		c1_sdram_we_n     => sdram_we_n ,
		c1_sdram_ba       => sdram_ba   ,
		c1_sdram_addr     => sdram_addr ,
		c1_sdram_dqm      => sdram_dqm  ,
		c1_sdram_dq       => sdram_dq   
	);
	    
end bhv;